#ifndef __G65DDCAPI_H__
#define __G65DDCAPI_H__

#ifdef WIN32_LEAN_AND_MEAN
#include <basetyps.h>
#endif //WIN32_LEAN_AND_MEAN

#include <windows.h>

#pragma pack(push,1)

#define G65DDC_CLASS_ID_DEVICE_ENUMERATOR               0x0000F001
#define G65DDC_CLASS_ID_DEVICE                          0x0000F002
#define G65DDC_CLASS_ID_NETWORK_ENUMERATOR              0x0000F003
#define G65DDC_CLASS_ID_COH_DEVICE_SET_ENUMERATOR       0x0000F804
#define G65DDC_CLASS_ID_COH_DEVICE_SET                  0x0000F805

typedef BOOL (__stdcall *G65DDC_CREATE_INSTANCE)(UINT32 ClassId,VOID **Interface);

#define G65DDC_OPEN_DEMO                                ((char*)0)
#define G65DDC_OPEN_FIRST                               ((char*)-1)

#define COH_G65DDC_OPEN_DEMO                            ((G65DDC_DEVICE_INFO*)0)
#define COH_G65DDC_OPEN_FIRST                           ((G65DDC_DEVICE_INFO*)-1)

#define COH_G65DDC_CONFIGURATION_NORMAL                 0
#define COH_G65DDC_CONFIGURATION_EXTENDED               1

#define G65DDC_INTERFACE_TYPE_PCIE                      0x00
#define G65DDC_INTERFACE_TYPE_USB2                      0x01
#define G65DDC_INTERFACE_TYPE_USB3                      0x02
#define G65DDC_INTERFACE_TYPE_LAN                       0x03
#define G65DDC_INTERFACE_TYPE_DEMO                      0xFF

#define G65DDC_FRONT_PANEL_LED_MODE_OFF                 0
#define G65DDC_FRONT_PANEL_LED_MODE_ON                  1
#define G65DDC_FRONT_PANEL_LED_MODE_DIAG                2

#define G65DDC_RANGE_1                                  0
#define G65DDC_RANGE_2                                  1
#define G65DDC_RANGE_COUNT                              2

#define G65DDC_DEVICE_STATE_HIGH_TEMPERATURE            0x00000001
//Coherent mode
#define G65DDC_DEVICE_STATE_NO_1PPS                     0x00010000
#define G65DDC_DEVICE_STATE_NOT_ARMED                   0x00020000
#define G65DDC_DEVICE_STATE_TRIGGER_NOT_SENT            0x00040000
#define G65DDC_DEVICE_STATE_NOT_TRIGGERED               0x00080000

#define G65DDC_TEMPERATURE_NA                           (-0x4000)

#define G65DDC_MODE_AM                                  0
#define G65DDC_MODE_AMS                                 1
#define G65DDC_MODE_LSB                                 2
#define G65DDC_MODE_USB                                 3
#define G65DDC_MODE_DSB                                 4
#define G65DDC_MODE_ISB                                 5
#define G65DDC_MODE_CW                                  6
#define G65DDC_MODE_FM                                  7
#define G65DDC_MODE_DRM                                 8
#define G65DDC_MODE_COUNT                               (G65DDC_MODE_DRM+1)

#define G65DDC_SIDE_BAND_LOWER                          1
#define G65DDC_SIDE_BAND_UPPER                          2
#define G65DDC_SIDE_BAND_BOTH                           3

#define G65DDC_AUDIO_STREAM_CALLBACK_STAGE_0            0
#define G65DDC_AUDIO_STREAM_CALLBACK_STAGE_1            1
#define G65DDC_AUDIO_STREAM_CALLBACK_STAGE_2            2

#define G65DDC_DEMODULATOR_PARAM_AMS_SIDE_BAND          0x00000001
#define G65DDC_DEMODULATOR_PARAM_AMS_CAPTURE_RANGE      0x00000002
#define G65DDC_DEMODULATOR_PARAM_CW_FREQUENCY           0x00000003
#define G65DDC_DEMODULATOR_PARAM_DSB_SIDE_BAND          0x00000004
#define G65DDC_DEMODULATOR_PARAM_ISB_SIDE_BAND          0x00000005
#define G65DDC_DEMODULATOR_PARAM_DRM_AUDIO_SERVICE      0x00000006
#define G65DDC_DEMODULATOR_PARAM_DRM_MULTIMEDIA_SERVICE 0x00000007

#define G65DDC_DEMODULATOR_STATE_AMS_LOCK               0x00000001
#define G65DDC_DEMODULATOR_STATE_AMS_FREQUENCY          0x00000002
#define G65DDC_DEMODULATOR_STATE_AM_DEPTH               0x00000003
#define G65DDC_DEMODULATOR_STATE_TUNE_ERROR             0x00000004
#define G65DDC_DEMODULATOR_STATE_FM_DEVIATION           0x00000005
#define G65DDC_DEMODULATOR_STATE_DSB_LOCK               0x00000006
#define G65DDC_DEMODULATOR_STATE_DSB_FREQUENCY          0x00000007
#define G65DDC_DEMODULATOR_STATE_DRM_STATUS             0x00000008

#define G65DDC_ADC_SAMPLES_PER_SNAPSHOT_64K             0x10000
#define G65DDC_ADC_SAMPLES_PER_SNAPSHOT_128K            0x20000
#define G65DDC_ADC_SAMPLES_PER_SNAPSHOT_256K            0x40000

#define G65DDC_TEMPERATURE_UNKNOWN                      -16384

#define G65DDC_FLAGS_EXTERNAL_REFERENCE_IN              0x00000001
#define G65DDC_FLAGS_COHERENT                           0x00000004
#define G65DDC_FLAGS_EXTERNAL_REFERENCE_OUT             0x00000008
#define G65DDC_FLAGS_1PPS                               0x00000010


typedef struct
{
    char        DevicePath[MAX_PATH];
    BYTE        InterfaceType;
    char        SerialNumber[9];
    WORD        HWVersion;    
    WORD        FWVersion[3];
    BYTE        EEPROMVersion;

    struct
    {
        UINT32  MinFrequency;
        UINT32  MaxFrequency;
    }           Ranges[G65DDC_RANGE_COUNT];    
        
    UINT32      MaxDDC1ChannelCount;    
    UINT32      MaxDDC2ChannelCount;
    UINT32      MaxDDC1TypeCount;
    UINT32      MaxDDC2TypeCount;   
    UINT32      MaxDDC2ChannelsPerDDC1Channel;

    UINT32      Flags;

    BYTE        MACAddress[6];
} G65DDC_DEVICE_INFO;

typedef struct
{
    UINT32      Flags;
    INT32       Temperatures[3];
    UINT32      FanRPM;
    UINT64      DataTransferred;
    UINT64      DataLost;
} G65DDC_DEVICE_STATE;

typedef struct
{
    UINT32  SampleRate;
    UINT32  Bandwidth;
    UINT32  BitsPerSample;
} G65DDC_DDC_INFO;

typedef struct
{
    UINT32  Tune;   //Hz
    UINT32  Lock;   //Hz
} G65DDC_AMS_CAPTURE_RANGE;

typedef struct
{
    WORD    Size;    
    UINT32  IpAddress;  //network-byte-order
    UINT32  Mask;       //network-byte-order
} G65DDC_NETWORK_PARAMS;

#define G65DDC_DRM_STATE_MODE_NOT_DETERMINED_YET       -1
#define G65DDC_DRM_STATE_MODE_A                        0
#define G65DDC_DRM_STATE_MODE_B                        1
#define G65DDC_DRM_STATE_MODE_C                        2
#define G65DDC_DRM_STATE_MODE_D                        3

#define G65DDC_DRM_STATE_INTERLEAVER_LONG              0       //long interleaver used (2 sec)
#define G65DDC_DRM_STATE_INTERLEAVER_SHORT             1       //short interleaver used (400 msec)

#define G65DDC_DRM_STATE_QAM_TYPE_STD                  0       //standard
#define G65DDC_DRM_STATE_QAM_TYPE_HIER_SYM             1       //hierarchical symmetrical
#define G65DDC_DRM_STATE_QAM_TYPE_HIER_MIX             2       //hierarchical mixed

#define G65DDC_DRM_STATE_SERVICE_CONTENT_EMPTY         0x00    //service is not used/contains no data
#define G65DDC_DRM_STATE_SERVICE_CONTENT_AUDIO         0x01    //service contains audio data
#define G65DDC_DRM_STATE_SERVICE_CONTENT_TEXTMSG       0x02    //service contains text messages
#define G65DDC_DRM_STATE_SERVICE_CONTENT_MULTIMEDIA    0x04    //service contains multimedia data
#define G65DDC_DRM_STATE_SERVICE_CONTENT_DATA          0x08    //service contains application specific data

#define G65DDC_DRM_STATE_AUDIO_CODING_AAC              0       //audio coding is AAC
#define G65DDC_DRM_STATE_AUDIO_CODING_CELP             1       //audio coding is CELP
#define G65DDC_DRM_STATE_AUDIO_CODING_HVXC             2       //audio coding is HVXC
#define G65DDC_DRM_STATE_AUDIO_CODING_RFU              3       //reserved for future use

#define G65DDC_DRM_STATE_AUDIO_MODE_AAC_MONO           0       //mono
#define G65DDC_DRM_STATE_AUDIO_MODE_AAC_PARAM_STEREO   1       //parametric stereo
#define G65DDC_DRM_STATE_AUDIO_MODE_AAC_STEREO         2       //stereo
#define G65DDC_DRM_STATE_AUDIO_MODE_AAC_RFU            3       //reserved for future use
	
#define G65DDC_DRM_STATE_AUDIO_MODE_CELP_NO_CRC        0       //audio data is without CRC
#define G65DDC_DRM_STATE_AUDIO_MODE_CELP_CRC           1       //CRC used
#define G65DDC_DRM_STATE_AUDIO_MODE_CELP_RFU_10        2       //reserved for future use
#define G65DDC_DRM_STATE_AUDIO_MODE_CELP_RFU_11        3       //reserved for future use
	
#define G65DDC_DRM_STATE_AUDIO_MODE_HVXC_RFU_00        0       //reserved for future use
#define G65DDC_DRM_STATE_AUDIO_MODE_HVXC_RFU_01        1       //reserved for future use
#define G65DDC_DRM_STATE_AUDIO_MODE_HVXC_RFU_10        2       //reserved for future use
#define G65DDC_DRM_STATE_AUDIO_MODE_HVXC_RFU_11        3       //reserved for future use

typedef struct
{
    BOOL            Valid;                          //entries below are invalid/valid

    struct
    {
        BOOL        SyncFound;
        BOOL        FACDecoded;
        BOOL        SDCDecoded;
        BOOL        AudioDecoded;
        SHORT       NumberOfAudioFrames;            //-1 if not available
        SHORT       NumberOfAudioErrors;            //-1 if not available	
    } DecodingState;


    INT32           Mode;                           //G65DDC_DRM_STATE_MODE_*
	double          RFBandwidth;                    //occupied RF bandwith in kHz, 0 = invalid or information not (yet) available
    BYTE            Interleaver;                    //G65DDC_DRM_STATE_INTERLEAVER_*
    SHORT           SDCQam;                         //coding of the SDC (QAM), 0 = not (yet) available
    SHORT           MSCQam;                         //coding of the MSC (QAM), 0 = not (yet) available
    BYTE            MSCQamType;                     //G65DDC_DRM_STATE_QAM_TYPE_* , used QAM coding for the MSC
    double          CoderateH;				        //used coderate for hierachical coding,  values <= 0 indicate not available or not used
    double          CoderateA;				        //used coderate for protection level A,  values <= 0 indicate not available or not used
    double          CoderateB;				        //used coderate for protection level B,  values <= 0 indicate not available or not used
    double          EstimatedSNR;			        //estimated SNR in dB of the decoded signal
    WCHAR           TextMessage[128 + 1 + 16];      //text message containing max. 128 byte text + string termination + max. 16 linebreaks
   
    struct
    {
        BYTE        Content;				        //G65DDC_DRM_STATE_SERVICE_CONTENT_*
        WCHAR       DynamicLabel[256];		        //string containing dynamic label of the service, zero terminated
        WCHAR       Country[256];			        //string containing the signaled country for this service
        WCHAR       Language[256];			        //string containing the signaled language for this service
        WCHAR       ProgramType[256];		        //string containing the signaled program type for this service
        double      AudioBitrate;			        //data rate for the audio content (0 if not available)
        double      TextMsgBitrate;			        //data rate for the text message content (0 if not available)
        double      MultimediaBitrate;		        //data rate for the multimedia content (0 if not available)
        double      DataBitrate;			        //data rate for the data content (0 if not available)
    } ServiceInfo[4];

    struct
    {
        BOOL        Valid;                          //audio decoder information valid/invalid, when invalid all other entries of the struct contain no valid data
        BYTE        AudioCoding;                    //G65DDC_DRM_STATE_AUDIO_CODING_*        
        BOOL        SBR;                            //SBR used/not used (TRUE/FALSE)
        INT32       AudioMode;                      //depeding on the audio coding, G65DDC_DRM_STATE_AUDIO_MODE_*
    } AudioDecoderInfo[4];
} G65DDC_DRM_STATUS;

#undef INTERFACE
#define INTERFACE IG65DDCDeviceEnumerator

DECLARE_INTERFACE(IG65DDCDeviceEnumerator)
{
    STDMETHOD_(void,Free)(THIS) PURE;

    STDMETHOD_(BOOL,Enumerate)(THIS) PURE;
    STDMETHOD_(UINT32,GetCount)(THIS) PURE;
    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ UINT32 DeviceIndex,G65DDC_DEVICE_INFO *DeviceInfo) PURE;
};


#undef INTERFACE
#define INTERFACE IG65DDCNetworkEnumeratorCallback

DECLARE_INTERFACE(IG65DDCNetworkEnumeratorCallback)
{
    STDMETHOD_(void,G65DDC_DeviceFound)(THIS_ UINT32 IpAddress,WORD Port,const char *SerialNumber,UINT32 Flags) PURE;
};

#undef INTERFACE
#define INTERFACE IG65DDCNetworkEnumerator

DECLARE_INTERFACE(IG65DDCNetworkEnumerator)
{
    STDMETHOD_(void,Free)(THIS) PURE;
    STDMETHOD_(BOOL,Start)(THIS_ IG65DDCNetworkEnumeratorCallback *Callback) PURE;
    STDMETHOD_(void,Stop)(THIS) PURE;
};

struct IG65DDCDevice;

#undef INTERFACE
#define INTERFACE IG65DDCDeviceCallback

DECLARE_INTERFACE(IG65DDCDeviceCallback)
{    
    STDMETHOD_(void,G65DDC_ADCSnapshotCallback)(THIS_ struct IG65DDCDevice *Device,const short *Buffer,UINT32 Count,UINT32 CenterFrequency,WORD ADCLevel) PURE;
    STDMETHOD_(void,G65DDC_DDC1StreamCallback)(THIS_ struct IG65DDCDevice *Device,UINT32 DDC1ChannelId,const void *Buffer,UINT32 Count,UINT32 BitsPerSample) PURE;
    STDMETHOD_(BOOL,G65DDC_DDC1PlaybackStreamCallback)(THIS_ struct IG65DDCDevice *Device,UINT32 DDC1ChannelId,void *Buffer,UINT32 Count,UINT32 BitsPerSample) PURE;
    STDMETHOD_(void,G65DDC_DDC2StreamCallback)(THIS_ struct IG65DDCDevice *Device,UINT32 DDC2ChannelId,const float *Buffer,UINT32 Count) PURE;
    STDMETHOD_(void,G65DDC_DDC2PreprocessedStreamCallback)(THIS_ struct IG65DDCDevice *Device,UINT32 DDC2ChannelId,const float *Buffer,UINT32 Count,float Peak,float RMS) PURE;
    STDMETHOD_(void,G65DDC_AudioStreamCallback)(THIS_ struct IG65DDCDevice *Device,UINT32 DDC2ChannelId,UINT32 Stage,const float *Buffer,UINT32 Count) PURE;
    STDMETHOD_(BOOL,G65DDC_AudioPlaybackStreamCallback)(THIS_ struct IG65DDCDevice *Device,UINT32 DDC2ChannelId,float *Buffer,UINT32 Count) PURE;
};

#define G65DDC_DEVICE_INTERFACE_VERSION         0x00000002

#undef INTERFACE
#define INTERFACE IG65DDCDevice

DECLARE_INTERFACE(IG65DDCDevice)
{
    STDMETHOD_(BOOL,Free)(THIS) PURE;

    STDMETHOD_(void,SetCallback)(THIS_ IG65DDCDeviceCallback *Callback) PURE;
    STDMETHOD_(IG65DDCDeviceCallback*,GetCallback)(THIS) PURE;

    STDMETHOD_(BOOL,Open)(THIS_ const char *Path) PURE;
    STDMETHOD_(BOOL,Close)(THIS) PURE;

    STDMETHOD_(BOOL,IsOpen)(THIS) PURE;

    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ G65DDC_DEVICE_INFO *DeviceInfo) PURE;
    
    STDMETHOD_(BOOL,IsConnected)(THIS_ BOOL *Connected) PURE;

    STDMETHOD_(BOOL,SetLED)(THIS_ UINT32 LEDMode) PURE;
    STDMETHOD_(BOOL,GetLED)(THIS_ UINT32 *LEDMode) PURE;

    STDMETHOD_(BOOL,SetPower)(THIS_ BOOL Power) PURE;
    STDMETHOD_(BOOL,GetPower)(THIS_ BOOL *Power) PURE;    
    
    STDMETHOD_(BOOL,GetDeviceState)(THIS_ G65DDC_DEVICE_STATE *State) PURE;

    STDMETHOD_(BOOL,SetExternalReference)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetExternalReference)(THIS_ BOOL *Enabled) PURE;

    STDMETHOD_(BOOL,SetBuiltInTest)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetBuiltInTest)(THIS_ BOOL *Enabled) PURE;

    STDMETHOD_(BOOL,SetAttenuator)(THIS_ UINT32 Attenuator) PURE;
    STDMETHOD_(BOOL,GetAttenuator)(THIS_ UINT32 *Attenuator) PURE;

    STDMETHOD_(BOOL,SetPreselectors)(THIS_ UINT32 Low,UINT32 High) PURE;
    STDMETHOD_(BOOL,GetPreselectors)(THIS_ UINT32 *Low,UINT32 *High) PURE;

    STDMETHOD_(BOOL,SetPreamplifier)(THIS_ BOOL Preamp) PURE;
    STDMETHOD_(BOOL,GetPreamplifier)(THIS_ BOOL *Preamp) PURE;    

    STDMETHOD_(BOOL,SetInverted)(THIS_ BOOL Inverted) PURE;
    STDMETHOD_(BOOL,GetInverted)(THIS_ BOOL *Inverted) PURE; 

    STDMETHOD_(BOOL,SetRange)(THIS_ UINT32 Range) PURE;
    STDMETHOD_(BOOL,GetRange)(THIS_ UINT32 *Range) PURE;

    STDMETHOD_(BOOL,SetADCNoiseBlanker)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetADCNoiseBlanker)(THIS_ BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetADCNoiseBlankerThreshold)(THIS_ WORD Threshold) PURE;
    STDMETHOD_(BOOL,GetADCNoiseBlankerThreshold)(THIS_ WORD *Threshold) PURE;

    STDMETHOD_(BOOL,StartADCSnapshots)(THIS_ WORD Interval,UINT32 SamplesPerSnapshot) PURE;
    STDMETHOD_(BOOL,StopADCSnapshots)(THIS) PURE;

    STDMETHOD_(BOOL,GetDDCInfo)(THIS_ UINT32 DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo) PURE;    

    STDMETHOD_(BOOL,CreateDDC1)(THIS_ UINT32 *DDC1ChannelId) PURE;
    STDMETHOD_(BOOL,CreateDDC2)(THIS_ UINT32 DDC1ChannelId,UINT32 *DDC2ChannelId) PURE;

    STDMETHOD_(BOOL,DeleteDDC)(THIS_ UINT32 DDCChannelId) PURE;
    
    STDMETHOD_(BOOL,GetDDCCount)(THIS_ UINT32 DDCChannelId,UINT32 *Count) PURE;
    STDMETHOD_(BOOL,SetDDC)(THIS_ UINT32 DDCChannelId,UINT32 TypeIndex) PURE;
    STDMETHOD_(BOOL,GetDDC)(THIS_ UINT32 DDCChannelId,UINT32 *TypeIndex,G65DDC_DDC_INFO *DDCInfo) PURE;
    STDMETHOD_(BOOL,StartDDC)(THIS_ UINT32 DDCChannelId,UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,StopDDC)(THIS_ UINT32 DDCChannelId) PURE;

    STDMETHOD_(BOOL,StartDDC1Playback)(THIS_ UINT32 DDC1ChannelId,UINT32 SampleSetsPerBuffer,UINT32 BitsPerSample) PURE;
    STDMETHOD_(BOOL,PauseDDC1Playback)(THIS_ UINT32 DDC1ChannelId) PURE;
    STDMETHOD_(BOOL,ResumeDDC1Playback)(THIS_ UINT32 DDC1ChannelId) PURE;

    STDMETHOD_(BOOL,SetDDCFrequency)(THIS_ UINT32 DDCChannelId,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetDDCFrequency)(THIS_ UINT32 DDCChannelId,INT32 *Frequency) PURE;

    STDMETHOD_(BOOL,SetDDC2NoiseBlanker)(THIS_ UINT32 DDC2ChannelId,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetDDC2NoiseBlanker)(THIS_ UINT32 DDC2ChannelId,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetDDC2NoiseBlankerThreshold)(THIS_ UINT32 DDC2ChannelId,double Threshold) PURE;
    STDMETHOD_(BOOL,GetDDC2NoiseBlankerThreshold)(THIS_ UINT32 DDC2ChannelId,double *Threshold) PURE;
    STDMETHOD_(BOOL,GetDDC2NoiseBlankerExcessValue)(THIS_ UINT32 DDC2ChannelId,double *Value) PURE;

    STDMETHOD_(BOOL,SetNotchFilter)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetNotchFilter)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetNotchFilterFrequency)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetNotchFilterFrequency)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,INT32 *Frequency) PURE;
    STDMETHOD_(BOOL,SetNotchFilterBandwidth)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 Bandwidth) PURE;
    STDMETHOD_(BOOL,GetNotchFilterBandwidth)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 *Bandwidth) PURE;
    STDMETHOD_(BOOL,SetNotchFilterLength)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetNotchFilterLength)(THIS_ UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 *Length) PURE;

    STDMETHOD_(BOOL,SetAGC)(THIS_ UINT32 DDC2ChannelId,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetAGC)(THIS_ UINT32 DDC2ChannelId,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetAGCParams)(THIS_ UINT32 DDC2ChannelId,double AttackTime,double DecayTime,double ReferenceLevel) PURE;
    STDMETHOD_(BOOL,GetAGCParams)(THIS_ UINT32 DDC2ChannelId,double *AttackTime,double *DecayTime,double *ReferenceLevel) PURE;
    STDMETHOD_(BOOL,SetMaxAGCGain)(THIS_ UINT32 DDC2ChannelId,double MaxGain) PURE;
    STDMETHOD_(BOOL,GetMaxAGCGain)(THIS_ UINT32 DDC2ChannelId,double *MaxGain) PURE;
    STDMETHOD_(BOOL,SetGain)(THIS_ UINT32 DDC2ChannelId,double Gain) PURE;
    STDMETHOD_(BOOL,GetGain)(THIS_ UINT32 DDC2ChannelID,double *Gain) PURE;
    STDMETHOD_(BOOL,GetCurrentGain)(THIS_ UINT32 DDC2ChannelId,double *Gain) PURE;

    STDMETHOD_(BOOL,GetSignalLevel)(THIS_ UINT32 DDC2ChannelId,float *Peak,float *RMS) PURE;

    STDMETHOD_(BOOL,SetDemodulatorMode)(THIS_ UINT32 DDC2ChannelId,UINT32 Mode) PURE;
    STDMETHOD_(BOOL,GetDemodulatorMode)(THIS_ UINT32 DDC2ChannelId,UINT32 *Mode) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFrequency)(THIS_ UINT32 DDC2ChannelId,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFrequency)(THIS_ UINT32 DDC2ChannelId,INT32 *Frequency) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFilterBandwidth)(THIS_ UINT32 DDC2ChannelId,UINT32 Bandwidth) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFilterBandwidth)(THIS_ UINT32 DDC2ChannelId,UINT32 *Bandwidth) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFilterShift)(THIS_ UINT32 DDC2ChannelId,INT32 Shift) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFilterShift)(THIS_ UINT32 DDC2ChannelId,INT32 *Shift) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFilterLength)(THIS_ UINT32 DDC2ChannelId,UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFilterLength)(THIS_ UINT32 DDC2ChannelId,UINT32 *Length) PURE;

    STDMETHOD_(BOOL,SetDemodulatorParam)(THIS_ UINT32 DDC2ChannelId,UINT32 Code,const void *Buffer,UINT32 BufferSize) PURE;
    STDMETHOD_(BOOL,GetDemodulatorParam)(THIS_ UINT32 DDC2ChannelId,UINT32 Code,void *Buffer,UINT32 BufferSize) PURE;
    STDMETHOD_(BOOL,GetDemodulatorState)(THIS_ UINT32 DDC2ChannelId,UINT32 Code,void *Buffer,UINT32 BufferSize) PURE;

    STDMETHOD_(BOOL,StartAudio)(THIS_ UINT32 DDC2ChannelId,UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,StopAudio)(THIS_ UINT32 DDC2ChannelId) PURE;

    STDMETHOD_(BOOL,StartAudioPlayback)(THIS_ UINT32 DDC2ChannelId,UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,PauseAudioPlayback)(THIS_ UINT32 DDC2ChannelId) PURE;
    STDMETHOD_(BOOL,ResumeAudioPlayback)(THIS_ UINT32 DDC2ChannelId) PURE;

    STDMETHOD_(BOOL,SetAudioFilter)(THIS_ UINT32 DDC2ChannelId,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetAudioFilter)(THIS_ UINT32 DDC2ChannelId,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetAudioFilterParams)(THIS_ UINT32 DDC2ChannelId,UINT32 CutOffLow,UINT32 CutOffHigh,double Deemphasis) PURE;    
    STDMETHOD_(BOOL,GetAudioFilterParams)(THIS_ UINT32 DDC2ChannelId,UINT32 *CutOffLow,UINT32 *CutOffHigh,double *Deemphasis) PURE;    
    STDMETHOD_(BOOL,SetAudioFilterLength)(THIS_ UINT32 DDC2ChannelId,UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetAudioFilterLength)(THIS_ UINT32 DDC2ChannelId,UINT32 *Length) PURE;
    STDMETHOD_(BOOL,SetAudioGain)(THIS_ UINT32 DDC2ChannelId,double Gain) PURE;
    STDMETHOD_(BOOL,GetAudioGain)(THIS_ UINT32 DDC2ChannelId,double *Gain) PURE;  
    STDMETHOD_(BOOL,SetVolume)(THIS_ UINT32 DDC2ChannelId,BYTE Volume) PURE;
    STDMETHOD_(BOOL,GetVolume)(THIS_ UINT32 DDC2ChannelId,BYTE *Volume) PURE;    
    STDMETHOD_(BOOL,SetMute)(THIS_ UINT32 DDC2ChannelId,BOOL Mute) PURE;
    STDMETHOD_(BOOL,GetMute)(THIS_ UINT32 DDC2ChannelId,BOOL *Mute) PURE;

    STDMETHOD_(BOOL,SetDRMKey)(THIS_ const char *DRMKeyFileDirectory) PURE;
    STDMETHOD_(BOOL,IsDRMUnlocked)(THIS) PURE;

    STDMETHOD_(BOOL,SetFrequency)(THIS_ UINT32 DDC2ChannelId,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetFrequency)(THIS_ UINT32 DDC2ChannelId,INT32 *Frequency) PURE;

    STDMETHOD_(BOOL,GetSpectrumCompensation)(THIS_ INT32 Frequency,UINT32 Bandwidth,float *Buffer,UINT32 Count) PURE;

    STDMETHOD_(BOOL,SetNetworkParams)(THIS_ const G65DDC_NETWORK_PARAMS *Params) PURE;
    STDMETHOD_(BOOL,GetNetworkParams)(THIS_ G65DDC_NETWORK_PARAMS *Params) PURE;

    STDMETHOD_(BOOL,GetVersion)(THIS_ UINT64 *Version) PURE;
    STDMETHOD_(BOOL,GetInterfaceVersion)(THIS_ UINT32 *Version) PURE;    

    STDMETHOD_(BOOL,Get1PPSCounters)(THIS_ UINT32 *EventCounter,UINT32 *ADCPeriodCounter) PURE;
    STDMETHOD_(BOOL,Get1PPSDDC1Counters)(THIS_ UINT32 DDC1ChannelId,double *SampleCounter,UINT64 *ADCPeriodCounter) PURE;

    STDMETHOD_(BOOL,SetExternalADCClock)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetExternalADCClock)(THIS_ BOOL *Enabled) PURE;
};

#undef INTERFACE
#define INTERFACE ICohG65DDCDeviceSetEnumerator

DECLARE_INTERFACE(ICohG65DDCDeviceSetEnumerator)
{
    STDMETHOD_(void,Free)(THIS) PURE;
    STDMETHOD_(BOOL,Enumerate)(THIS) PURE;
    STDMETHOD_(UINT32,GetDeviceSetCount)(THIS) PURE;
    STDMETHOD_(BOOL,GetDeviceSetInfo)(THIS_ UINT32 DeviceSetIndex,G65DDC_DEVICE_INFO *DeviceInfos,UINT32 *DeviceInfoCount) PURE;
};     


#undef INTERFACE
#define INTERFACE ICohG65DDCDeviceSetCallback

struct ICohG65DDCDeviceSet;

DECLARE_INTERFACE(ICohG65DDCDeviceSetCallback)
{
    STDMETHOD_(void,CohG65DDC_ADCSnapshotCallback)(THIS_ struct ICohG65DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,const short *Buffer,UINT32 Count,UINT32 CenterFrequency,WORD ADCLevel) PURE;
    STDMETHOD_(void,CohG65DDC_DDC1StreamCallback)(THIS_ struct ICohG65DDCDeviceSet *DeviceSet,UINT32 DeviceCount,const void **Buffers,UINT32 Count,UINT32 BitsPerSample) PURE;    
    STDMETHOD_(void,CohG65DDC_DDC2StreamCallback)(THIS_ struct ICohG65DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,const float *Buffer,UINT32 Count) PURE;
    STDMETHOD_(void,CohG65DDC_DDC2PreprocessedStreamCallback)(THIS_ struct ICohG65DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,const float *Buffer,UINT32 Count,float SlevelPeak,float SlevelRMS) PURE;
    STDMETHOD_(void,CohG65DDC_AudioStreamCallback)(THIS_ struct ICohG65DDCDeviceSet *DeviceSet,UINT32 DeviceIndex,UINT32 Stage,const float *Buffer,UINT32 Count) PURE;    
};

#define COH_G65DDC_DEVICE_SET_INTERFACE_VERSION     0x00000001

#undef INTERFACE
#define INTERFACE ICohG65DDCDeviceSet

DECLARE_INTERFACE(ICohG65DDCDeviceSet)
{
    STDMETHOD_(void,Free)(THIS) PURE;

    STDMETHOD_(void,SetCallback)(THIS_ ICohG65DDCDeviceSetCallback *Callback) PURE;
    STDMETHOD_(ICohG65DDCDeviceSetCallback*,GetCallback)(THIS) PURE;

    STDMETHOD_(BOOL,Open)(THIS_ G65DDC_DEVICE_INFO *DeviceInfos,UINT32 Count) PURE;
    STDMETHOD_(BOOL,Close)(THIS) PURE;

    STDMETHOD_(BOOL,IsOpen)(THIS) PURE;

    STDMETHOD_(BOOL,GetDeviceCount)(THIS_ UINT32 *Count) PURE;

    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ UINT32 DeviceIndex,G65DDC_DEVICE_INFO *DeviceInfo) PURE;
    STDMETHOD_(BOOL,GetDeviceInfos)(THIS_ G65DDC_DEVICE_INFO *Infos,UINT32 *InfoCount) PURE;
    
    STDMETHOD_(BOOL,IsConnected)(THIS_ BOOL *Connected) PURE;

    STDMETHOD_(BOOL,SetLED)(THIS_ UINT32 DeviceIndex,UINT32 LEDMode) PURE;
    STDMETHOD_(BOOL,GetLED)(THIS_ UINT32 DeviceIndex,UINT32 *LEDMode) PURE;

    STDMETHOD_(BOOL,SetPower)(THIS_ BOOL Power) PURE;
    STDMETHOD_(BOOL,GetPower)(THIS_ BOOL *Power) PURE;
    
    STDMETHOD_(BOOL,GetDeviceState)(THIS_ UINT32 DeviceIndex,G65DDC_DEVICE_STATE *State) PURE;
    
    STDMETHOD_(BOOL,SetBuiltInTest)(THIS_ UINT32 DeviceIndex,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetBuiltInTest)(THIS_ UINT32 DeviceIndex,BOOL *Enabled) PURE;

    STDMETHOD_(BOOL,SetAttenuator)(THIS_ UINT32 Attenuator) PURE;
    STDMETHOD_(BOOL,GetAttenuator)(THIS_ UINT32 *Attenuator) PURE;

    STDMETHOD_(BOOL,SetPreselectors)(THIS_ UINT32 Low,UINT32 High) PURE;
    STDMETHOD_(BOOL,GetPreselectors)(THIS_ UINT32 *Low,UINT32 *High) PURE;

    STDMETHOD_(BOOL,SetPreamplifier)(THIS_ BOOL Preamp) PURE;
    STDMETHOD_(BOOL,GetPreamplifier)(THIS_ BOOL *Preamp) PURE;    

    STDMETHOD_(BOOL,SetInverted)(THIS_ BOOL Inverted) PURE;
    STDMETHOD_(BOOL,GetInverted)(THIS_ BOOL *Inverted) PURE; 

    STDMETHOD_(BOOL,SetRange)(THIS_ UINT32 Range) PURE;
    STDMETHOD_(BOOL,GetRange)(THIS_ UINT32 *Range) PURE;

    STDMETHOD_(BOOL,SetADCNoiseBlanker)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetADCNoiseBlanker)(THIS_ BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetADCNoiseBlankerThreshold)(THIS_ WORD Threshold) PURE;
    STDMETHOD_(BOOL,GetADCNoiseBlankerThreshold)(THIS_ WORD *Threshold) PURE;

    STDMETHOD_(BOOL,StartADCSnapshots)(THIS_ UINT32 DeviceIndex,WORD Interval,UINT32 SamplesPerSnapshot) PURE;
    STDMETHOD_(BOOL,StopADCSnapshots)(THIS_ UINT32 DeviceIndex) PURE;

    STDMETHOD_(BOOL,GetDDCInfo)(THIS_ UINT32 DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo) PURE;    

    STDMETHOD_(BOOL,GetDDC1Count)(THIS_ UINT32 *Count) PURE;
    STDMETHOD_(BOOL,SetDDC1)(THIS_ UINT32 DDCTypeIndex) PURE;
    STDMETHOD_(BOOL,GetDDC1)(THIS_ UINT32 *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo) PURE;
    STDMETHOD_(BOOL,StartDDC1)(THIS_ UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,StopDDC1)(THIS) PURE;  
    STDMETHOD_(BOOL,SetDDC1Frequency)(THIS_ INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetDDC1Frequency)(THIS_ INT32 *Frequency) PURE;
    STDMETHOD_(BOOL,SetDDC1PhaseShift)(THIS_ UINT32 DeviceIndex,double PhaseShift) PURE;
    STDMETHOD_(BOOL,GetDDC1PhaseShift)(THIS_ UINT32 DeviceIndex,double *PhaseShift) PURE;

    STDMETHOD_(BOOL,GetDDC2Count)(THIS_ UINT32 *Count) PURE;
    STDMETHOD_(BOOL,SetDDC2)(THIS_ UINT32 DeviceIndex,UINT32 DDCTypeIndex) PURE;
    STDMETHOD_(BOOL,GetDDC2)(THIS_ UINT32 DeviceIndex,UINT32 *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo) PURE;
    STDMETHOD_(BOOL,StartDDC2)(THIS_ UINT32 DeviceIndex,UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,StopDDC2)(THIS_ UINT32 DeviceIndex) PURE;  
    STDMETHOD_(BOOL,SetDDC2Frequency)(THIS_ UINT32 DeviceIndex,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetDDC2Frequency)(THIS_ UINT32 DeviceIndex,INT32 *Frequency) PURE;
    
    STDMETHOD_(BOOL,SetDDC2NoiseBlanker)(THIS_ UINT32 DeviceIndex,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetDDC2NoiseBlanker)(THIS_ UINT32 DeviceIndex,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetDDC2NoiseBlankerThreshold)(THIS_ UINT32 DeviceIndex,double Threshold) PURE;
    STDMETHOD_(BOOL,GetDDC2NoiseBlankerThreshold)(THIS_ UINT32 DeviceIndex,double *Threshold) PURE;
    STDMETHOD_(BOOL,GetDDC2NoiseBlankerExcessValue)(THIS_ UINT32 DeviceIndex,double *Value) PURE;

    STDMETHOD_(BOOL,SetNotchFilter)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetNotchFilter)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetNotchFilterFrequency)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetNotchFilterFrequency)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,INT32 *Frequency) PURE;
    STDMETHOD_(BOOL,SetNotchFilterBandwidth)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 Bandwidth) PURE;
    STDMETHOD_(BOOL,GetNotchFilterBandwidth)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 *Bandwidth) PURE;
    STDMETHOD_(BOOL,SetNotchFilterLength)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetNotchFilterLength)(THIS_ UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 *Length) PURE;

    STDMETHOD_(BOOL,SetAGC)(THIS_ UINT32 DeviceIndex,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetAGC)(THIS_ UINT32 DeviceIndex,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetAGCParams)(THIS_ UINT32 DeviceIndex,double AttackTime,double DecayTime,double ReferenceLevel) PURE;
    STDMETHOD_(BOOL,GetAGCParams)(THIS_ UINT32 DeviceIndex,double *AttackTime,double *DecayTime,double *ReferenceLevel) PURE;
    STDMETHOD_(BOOL,SetMaxAGCGain)(THIS_ UINT32 DeviceIndex,double MaxGain) PURE;
    STDMETHOD_(BOOL,GetMaxAGCGain)(THIS_ UINT32 DeviceIndex,double *MaxGain) PURE;
    STDMETHOD_(BOOL,SetGain)(THIS_ UINT32 DeviceIndex,double Gain) PURE;
    STDMETHOD_(BOOL,GetGain)(THIS_ UINT32 DeviceIndex,double *Gain) PURE;
    STDMETHOD_(BOOL,GetCurrentGain)(THIS_ UINT32 DeviceIndex,double *Gain) PURE;

    STDMETHOD_(BOOL,GetSignalLevel)(THIS_ UINT32 DeviceIndex,float *Peak,float *RMS) PURE;

    STDMETHOD_(BOOL,SetDemodulatorMode)(THIS_ UINT32 DeviceIndex,UINT32 Mode) PURE;
    STDMETHOD_(BOOL,GetDemodulatorMode)(THIS_ UINT32 DeviceIndex,UINT32 *Mode) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFrequency)(THIS_ UINT32 DeviceIndex,INT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFrequency)(THIS_ UINT32 DeviceIndex,INT32 *Frequency) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFilterBandwidth)(THIS_ UINT32 DeviceIndex,UINT32 Bandwidth) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFilterBandwidth)(THIS_ UINT32 DeviceIndex,UINT32 *Bandwidth) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFilterShift)(THIS_ UINT32 DeviceIndex,INT32 Shift) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFilterShift)(THIS_ UINT32 DeviceIndex,INT32 *Shift) PURE;
    STDMETHOD_(BOOL,SetDemodulatorFilterLength)(THIS_ UINT32 DeviceIndex,UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetDemodulatorFilterLength)(THIS_ UINT32 DeviceIndex,UINT32 *Length) PURE;

    STDMETHOD_(BOOL,SetDemodulatorParam)(THIS_ UINT32 DeviceIndex,UINT32 Code,const void *Buffer,UINT32 BufferSize) PURE;
    STDMETHOD_(BOOL,GetDemodulatorParam)(THIS_ UINT32 DeviceIndex,UINT32 Code,void *Buffer,UINT32 BufferSize) PURE;
    STDMETHOD_(BOOL,GetDemodulatorState)(THIS_ UINT32 DeviceIndex,UINT32 Code,void *Buffer,UINT32 BufferSize) PURE;

    STDMETHOD_(BOOL,StartAudio)(THIS_ UINT32 DeviceIndex,UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,StopAudio)(THIS_ UINT32 DeviceIndex) PURE;    

    STDMETHOD_(BOOL,SetAudioFilter)(THIS_ UINT32 DeviceIndex,BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetAudioFilter)(THIS_ UINT32 DeviceIndex,BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetAudioFilterParams)(THIS_ UINT32 DeviceIndex,UINT32 CutOffLow,UINT32 CutOffHigh,double Deemphasis) PURE;    
    STDMETHOD_(BOOL,GetAudioFilterParams)(THIS_ UINT32 DeviceIndex,UINT32 *CutOffLow,UINT32 *CutOffHigh,double *Deemphasis) PURE;    
    STDMETHOD_(BOOL,SetAudioFilterLength)(THIS_ UINT32 DeviceIndex,UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetAudioFilterLength)(THIS_ UINT32 DeviceIndex,UINT32 *Length) PURE;
    STDMETHOD_(BOOL,SetAudioGain)(THIS_ UINT32 DeviceIndex,double Gain) PURE;
    STDMETHOD_(BOOL,GetAudioGain)(THIS_ UINT32 DeviceIndex,double *Gain) PURE;  
    STDMETHOD_(BOOL,SetVolume)(THIS_ UINT32 DeviceIndex,BYTE Volume) PURE;
    STDMETHOD_(BOOL,GetVolume)(THIS_ UINT32 DeviceIndex,BYTE *Volume) PURE;    
    STDMETHOD_(BOOL,SetMute)(THIS_ UINT32 DeviceIndex,BOOL Mute) PURE;
    STDMETHOD_(BOOL,GetMute)(THIS_ UINT32 DeviceIndex,BOOL *Mute) PURE;

    STDMETHOD_(BOOL,SetDRMKey)(THIS_ const char *DRMKeyFileDirectory) PURE;
    STDMETHOD_(BOOL,IsDRMUnlocked)(THIS) PURE;
    
    STDMETHOD_(BOOL,GetSpectrumCompensation)(THIS_ UINT32 DeviceIndex,INT32 Frequency,UINT32 Bandwidth,float *Buffer,UINT32 Count) PURE;
    
    STDMETHOD_(BOOL,GetVersion)(THIS_ UINT64 *Version) PURE;
    STDMETHOD_(BOOL,GetInterfaceVersion)(THIS_ UINT32 *Version) PURE;    

    STDMETHOD_(BOOL,Set1PPSEnabled)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,Get1PPSEnabled)(THIS_ BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,Get1PPSCounters)(THIS_ UINT32 *EventCounter,UINT32 *ADCPeriodCounter) PURE;
    STDMETHOD_(BOOL,Get1PPSDDC1Counters)(THIS_ double *SampleCounter,UINT64 *ADCPeriodCounter) PURE;  

    STDMETHOD_(BOOL,SetTriggerDelay)(THIS_ UINT32 Delay) PURE;
    STDMETHOD_(BOOL,GetTriggerDelay)(THIS_ UINT32 *Delay) PURE;

    STDMETHOD_(BOOL,SetConfiguration)(THIS_ UINT32 Configuration) PURE;
    STDMETHOD_(BOOL,GetConfiguration)(THIS_ UINT32 *Configuration) PURE;
    STDMETHOD_(BOOL,SetExtendedConfigurationParams)(THIS_ UINT32 Index,UINT32 DeviceCount) PURE;
    STDMETHOD_(BOOL,GetExtendedConfigurationParams)(THIS_ UINT32 *Index,UINT32 *DeviceCount) PURE;
    STDMETHOD_(BOOL,Trigger)(THIS) PURE;
    STDMETHOD_(BOOL,TriggerTest)(THIS) PURE;
};

typedef void (__stdcall *G65DDC_ADC_SNAPSHOT_CALLBACK)(const short *Buffer,UINT32 Count,UINT32 CenterFrequency,WORD ADCLevel,DWORD_PTR UserData);
typedef void (__stdcall *G65DDC_DDC1_STREAM_CALLBACK)(UINT32 DDC1ChannelId,const void *Buffer,UINT32 Count,UINT32 BitsPerSample,DWORD_PTR UserData);
typedef BOOL (__stdcall *G65DDC_DDC1_PLAYBACK_STREAM_CALLBACK)(UINT32 DDC1ChannelId,void *Buffer,UINT32 Count,UINT32 BitsPerSample,DWORD_PTR UserData);
typedef void (__stdcall *G65DDC_DDC2_STREAM_CALLBACK)(UINT32 DDC2ChannelId,const float *Buffer,UINT32 Count,DWORD_PTR UserData);
typedef void (__stdcall *G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(UINT32 DDC2ChannelId,const float *Buffer,UINT32 Count,float Peak,float RMS,DWORD_PTR UserData);
typedef void (__stdcall *G65DDC_AUDIO_STREAM_CALLBACK)(UINT32 DDC2ChannelId,UINT32 Stage,const float *Buffer,UINT32 Count,DWORD_PTR UserData);
typedef BOOL (__stdcall *G65DDC_AUDIO_PLAYBACK_STREAM_CALLBACK)(UINT32 DDC2ChannelId,float *Buffer,UINT32 Count,DWORD_PTR UserData);
typedef BOOL (__stdcall *G65DDC_SEARCH_NETWORK_DEVICES_CALLBACK)(UINT32 IpAddress,WORD Port,const char *SerialNumber,UINT32 Flags,DWORD_PTR UserData);

typedef struct
{
    G65DDC_ADC_SNAPSHOT_CALLBACK                ADCSnapshotCallback;
    G65DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    G65DDC_DDC1_PLAYBACK_STREAM_CALLBACK        DDC1PlaybackStreamCallback;
    G65DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    G65DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
    G65DDC_AUDIO_PLAYBACK_STREAM_CALLBACK       AudioPlaybackStreamCallback;
} G65DDC_CALLBACKS;

#pragma pack(pop)

typedef BOOL (__stdcall *G65DDC_GET_DEVICE_LIST)(G65DDC_DEVICE_INFO **DeviceList,UINT32 *Count);
typedef void (__stdcall *G65DDC_FREE_DEVICE_LIST)(G65DDC_DEVICE_INFO *DeviceList);
typedef BOOL (__stdcall *G65DDC_SEARCH_NETWORK_DEVICES)(G65DDC_SEARCH_NETWORK_DEVICES_CALLBACK Callback,UINT_PTR UserData,UINT32 Timeout);
typedef INT32 (__stdcall *G65DDC_OPEN_DEVICE)(const char *SerialNumber);
typedef BOOL (__stdcall *G65DDC_CLOSE_DEVICE)(INT32 hDevice);
typedef BOOL (__stdcall *G65DDC_IS_DEVICE_CONNECTED)(INT32 hDevice,BOOL *Connected);
typedef BOOL (__stdcall *G65DDC_SET_CALLBACKS)(INT32 hDevice,const G65DDC_CALLBACKS *Callbacks,DWORD_PTR UserData);
typedef BOOL (__stdcall *G65DDC_GET_DEVICE_INFO)(INT32 hDevice,G65DDC_DEVICE_INFO *DeviceInfo);
typedef BOOL (__stdcall *G65DDC_SET_LED)(INT32 hDevice,UINT32 LEDMode);
typedef BOOL (__stdcall *G65DDC_GET_LED)(INT32 hDevice,UINT32 *LEDMode);
typedef BOOL (__stdcall *G65DDC_SET_POWER)(INT32 hDevice,BOOL Power);
typedef BOOL (__stdcall *G65DDC_GET_POWER)(INT32 hDevice,BOOL *Power);
typedef BOOL (__stdcall *G65DDC_GET_DEVICE_STATE)(INT32 hDevice,G65DDC_DEVICE_STATE *State);
typedef BOOL (__stdcall *G65DDC_SET_EXTERNAL_REFERENCE)(INT32 hDevice,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_EXTERNAL_REFERENCE)(INT32 hDevice,BOOL *Enabled);
typedef BOOL (__stdcall *G65DDC_SET_BUILT_IN_TEST)(INT32 hDevice,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_BUILT_IN_TEST)(INT32 hDevice,BOOL *Enabled);    
typedef BOOL (__stdcall *G65DDC_SET_ATTENUATOR)(INT32 hDevice,UINT32 Attenuator);
typedef BOOL (__stdcall *G65DDC_GET_ATTENUATOR)(INT32 hDevice,UINT32 *Attenuator);
typedef BOOL (__stdcall *G65DDC_SET_PRESELECTOS)(INT32 hDevice,UINT32 Low,UINT32 High);
typedef BOOL (__stdcall *G65DDC_GET_PRESELECTOS)(INT32 hDevice,UINT32 *Low,UINT32 *High);
typedef BOOL (__stdcall *G65DDC_SET_PREAMPLIFIER)(INT32 hDevice,BOOL Preamp);
typedef BOOL (__stdcall *G65DDC_GET_PREAMPLIFIER)(INT32 hDevice,BOOL *Preamp);
typedef BOOL (__stdcall *G65DDC_SET_RANGE)(INT32 hDevice,UINT32 Range);
typedef BOOL (__stdcall *G65DDC_GET_RANGE)(INT32 hDevice,UINT32 *Range);
typedef BOOL (__stdcall *G65DDC_SET_ADC_NOISE_BLANKER)(INT32 hDevice,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_ADC_NOISE_BLANKER)(INT32 hDevice,BOOL *Enabled);
typedef BOOL (__stdcall *G65DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(INT32 hDevice,WORD Threshold);
typedef BOOL (__stdcall *G65DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(INT32 hDevice,WORD *Threshold);
typedef BOOL (__stdcall *G65DDC_START_ADC_SNAPSHOTS)(INT32 hDevice,WORD Interval,UINT32 SamplesPerSnapshot);
typedef BOOL (__stdcall *G65DDC_STOP_ADC_SNAPSHOTS)(INT32 hDevice);
typedef BOOL (__stdcall *G65DDC_GET_DDC_INFO)(INT32 hDevice,UINT32 DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef BOOL (__stdcall *G65DDC_CREATE_DDC1)(INT32 hDevice,UINT32 *DDC1ChannelId);
typedef BOOL (__stdcall *G65DDC_CREATE_DDC2)(INT32 hDevice,UINT32 DDC1ChannelId,UINT32 *DDC2ChannelId);
typedef BOOL (__stdcall *G65DDC_DELETE_DDC)(INT32 hDevice,UINT32 DDCChannelId);
typedef BOOL (__stdcall *G65DDC_GET_DDC_COUNT)(INT32 hDevice,UINT32 DDCChannelId,UINT32 *Count);
typedef BOOL (__stdcall *G65DDC_SET_DDC)(INT32 hDevice,UINT32 DDCChannelId,UINT32 TypeIndex);
typedef BOOL (__stdcall *G65DDC_GET_DDC)(INT32 hDevice,UINT32 DDCChannelId,UINT32 *TypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef BOOL (__stdcall *G65DDC_START_DDC)(INT32 hDevice,UINT32 DDCChannelId,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *G65DDC_STOP_DDC)(INT32 hDevice,UINT32 DDCChannelId);
typedef BOOL (__stdcall *G65DDC_START_DDC1_PLAYBACK)(INT32 hDevice,UINT32 DDC1ChannelId,UINT32 SampleSetsPerBuffer,UINT32 BitsPerSample);
typedef BOOL (__stdcall *G65DDC_PAUSE_DDC1_PLAYBACK)(INT32 hDevice,UINT32 DDC1ChannelId);
typedef BOOL (__stdcall *G65DDC_RESUME_DDC1_PLAYBACK)(INT32 hDevice,UINT32 DDC1ChannelId);
typedef BOOL (__stdcall *G65DDC_SET_DDC_FREQUENCY)(INT32 hDevice,UINT32 DDCChannelId,INT32 Frequency);
typedef BOOL (__stdcall *G65DDC_GET_DDC_FREQUENCY)(INT32 hDevice,UINT32 DDCChannelId,INT32 *Frequency);
typedef BOOL (__stdcall *G65DDC_SET_DDC2_NOISE_BLANKER)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_DDC2_NOISE_BLANKER)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL *Enabled);
typedef BOOL (__stdcall *G65DDC_SET_DDC2_NOISE_BLANKER_THRESHOLD)(INT32 hDevice,UINT32 DDC2ChannelId,double Threshold);
typedef BOOL (__stdcall *G65DDC_GET_DDC2_NOISE_BLANKER_THRESHOLD)(INT32 hDevice,UINT32 DDC2ChannelId,double *Threshold);
typedef BOOL (__stdcall *G65DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(INT32 hDevice,UINT32 DDC2ChannelId,double *Value);
typedef BOOL (__stdcall *G65DDC_SET_NOTCH_FILTER)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_NOTCH_FILTER)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,BOOL *Enabled);
typedef BOOL (__stdcall *G65DDC_SET_NOTCH_FILTER_FREQUENCY)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,INT32 Frequency);
typedef BOOL (__stdcall *G65DDC_GET_NOTCH_FILTER_FREQUENCY)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,INT32 *Frequency);
typedef BOOL (__stdcall *G65DDC_SET_NOTCH_FILTER_BANDWIDTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 Bandwidth);
typedef BOOL (__stdcall *G65DDC_GET_NOTCH_FILTER_BANDWIDTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 *Bandwidth);
typedef BOOL (__stdcall *G65DDC_SET_NOTCH_FILTER_LENGTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 Length);
typedef BOOL (__stdcall *G65DDC_GET_NOTCH_FILTER_LENGTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 NotchFilterIndex,UINT32 *Length);
typedef BOOL (__stdcall *G65DDC_SET_AGC)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_AGC)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL *Enabled);
typedef BOOL (__stdcall *G65DDC_SET_AGC_PARAMS)(INT32 hDevice,UINT32 DDC2ChannelId,double AttackTime,double DecayTime,double ReferenceLevel);
typedef BOOL (__stdcall *G65DDC_GET_AGC_PARAMS)(INT32 hDevice,UINT32 DDC2ChannelId,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef BOOL (__stdcall *G65DDC_SET_MAX_AGC_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double MaxGain);
typedef BOOL (__stdcall *G65DDC_GET_MAX_AGC_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double *MaxGain);
typedef BOOL (__stdcall *G65DDC_SET_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double Gain);
typedef BOOL (__stdcall *G65DDC_GET_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double *Gain);
typedef BOOL (__stdcall *G65DDC_GET_CURRENT_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double *Gain);
typedef BOOL (__stdcall *G65DDC_GET_SIGNAL_LEVEL)(INT32 hDevice,UINT32 DDC2ChannelId,float *Peak,float *RMS);
typedef BOOL (__stdcall *G65DDC_SET_DEMODULATOR_MODE)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Mode);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_MODE)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 *Mode);
typedef BOOL (__stdcall *G65DDC_SET_DEMODULATOR_FREQUENCY)(INT32 hDevice,UINT32 DDC2ChannelId,INT32 Frequency);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_FREQUENCY)(INT32 hDevice,UINT32 DDC2ChannelId,INT32 *Frequency);
typedef BOOL (__stdcall *G65DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Bandwidth);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 *Bandwidth);
typedef BOOL (__stdcall *G65DDC_SET_DEMODULATOR_FILTER_SHIFT)(INT32 hDevice,UINT32 DDC2ChannelId,INT32 Shift);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_FILTER_SHIFT)(INT32 hDevice,UINT32 DDC2ChannelId,INT32 *Shift);
typedef BOOL (__stdcall *G65DDC_SET_DEMODULATOR_FILTER_LENGTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Length);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_FILTER_LENGTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 *Length);
typedef BOOL (__stdcall *G65DDC_SET_DEMODULATOR_PARAM)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Code,const void *Buffer,UINT32 BufferSize);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_PARAM)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Code,void *Buffer,UINT32 BufferSize);
typedef BOOL (__stdcall *G65DDC_GET_DEMODULATOR_STATE)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Code,void *Buffer,UINT32 BufferSize);
typedef BOOL (__stdcall *G65DDC_START_AUDIO)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *G65DDC_STOP_AUDIO)(INT32 hDevice,UINT32 DDC2ChannelId);
typedef BOOL (__stdcall *G65DDC_START_AUDIO_PLAYBACK)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *G65DDC_PAUSE_AUDIO_PLAYBACK)(INT32 hDevice,UINT32 DDC2ChannelId);
typedef BOOL (__stdcall *G65DDC_RESUME_AUDIO_PLAYBACK)(INT32 hDevice,UINT32 DDC2ChannelId);
typedef BOOL (__stdcall *G65DDC_SET_AUDIO_FILTER)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_AUDIO_FILTER)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL *Enabled);
typedef BOOL (__stdcall *G65DDC_SET_AUDIO_FILTER_PARAMS)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 CutOffLow,UINT32 CutOffHigh,double Deemphasis); 
typedef BOOL (__stdcall *G65DDC_GET_AUDIO_FILTER_PARAMS)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 *CutOffLow,UINT32 *CutOffHigh,double *Deemphasis); 
typedef BOOL (__stdcall *G65DDC_SET_AUDIO_FILTER_LENGTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 Length);
typedef BOOL (__stdcall *G65DDC_GET_AUDIO_FILTER_LENGTH)(INT32 hDevice,UINT32 DDC2ChannelId,UINT32 *Length);
typedef BOOL (__stdcall *G65DDC_SET_AUDIO_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double Gain);
typedef BOOL (__stdcall *G65DDC_GET_AUDIO_GAIN)(INT32 hDevice,UINT32 DDC2ChannelId,double *Gain);
typedef BOOL (__stdcall *G65DDC_SET_VOLUME)(INT32 hDevice,UINT32 DDC2ChannelId,BYTE Volume);
typedef BOOL (__stdcall *G65DDC_GET_VOLUME)(INT32 hDevice,UINT32 DDC2ChannelId,BYTE *Volume);
typedef BOOL (__stdcall *G65DDC_SET_MUTE)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL Mute);
typedef BOOL (__stdcall *G65DDC_GET_MUTE)(INT32 hDevice,UINT32 DDC2ChannelId,BOOL *Mute);
typedef BOOL (__stdcall *G65DDC_SET_DRM_KEY)(INT32 hDevice,const char *DRMKeyFileDirectory);
typedef BOOL (__stdcall *G65DDC_IS_DRM_UNLOCKED)(INT32 hDevice);
typedef BOOL (__stdcall *G65DDC_SET_FREQUENCY)(INT32 hDevice,UINT32 DDC2ChannelId,INT32 Frequency);
typedef BOOL (__stdcall *G65DDC_GET_FREQUENCY)(INT32 hDevice,UINT32 DDC2ChannelId,INT32 *Frequency);
typedef BOOL (__stdcall *G65DDC_GET_SPECTRUM_COMPENSATION)(INT32 hDevice,INT32 Frequency,UINT32 Bandwidth,float *Buffer,UINT32 Count);
typedef BOOL (__stdcall *G65DDC_SET_NETWORK_PARAMS)(INT32 hDevice,const G65DDC_NETWORK_PARAMS *Params);
typedef BOOL (__stdcall *G65DDC_GET_NETWORK_PARAMS)(INT32 hDevice,G65DDC_NETWORK_PARAMS *Params);
typedef BOOL (__stdcall *G65DDC_GET_1PPS_COUNTERS)(INT32 hDevice,UINT32 *EventCounter,UINT32 *ADCPeriodCounter);
typedef BOOL (__stdcall *G65DDC_GET_1PPS_DDC1_COUNTERS)(INT32 hDevice,UINT32 DDC1ChannelId,double *SampleCounter,UINT64 *ADCPeriodCounter);
typedef BOOL (__stdcall *G65DDC_SET_EXTERNAL_ADC_CLOCK)(INT32 hDevice,BOOL Enabled);
typedef BOOL (__stdcall *G65DDC_GET_EXTERNAL_ADC_CLOCK)(INT32 hDevice,BOOL *Enabled);


typedef void (__stdcall *COH_G65DDC_ADC_SNAPSHOT_CALLBACK)(UINT32 DeviceIndex,const short *Buffer,UINT32 Count,UINT32 CenterFrequency,WORD ADCLevel,DWORD_PTR UserData);
typedef void (__stdcall *COH_G65DDC_DDC1_STREAM_CALLBACK)(UINT32 DeviceCount,const void **Buffers,UINT32 Count,UINT32 BitsPerSample,DWORD_PTR UserData);
typedef void (__stdcall *COH_G65DDC_DDC2_STREAM_CALLBACK)(UINT32 DeviceIndex,const float *Buffer,UINT32 Count,DWORD_PTR UserData);
typedef void (__stdcall *COH_G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(UINT32 DeviceIndex,const float *Buffer,UINT32 Count,float SlevelPeak,float SlevelRMS,DWORD_PTR UserData);
typedef void (__stdcall *COH_G65DDC_AUDIO_STREAM_CALLBACK)(UINT32 DeviceIndex,UINT32 Stage,const float *Buffer,UINT32 Count,DWORD_PTR UserData); 

#pragma pack(push,1)

typedef struct
{
    COH_G65DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    COH_G65DDC_ADC_SNAPSHOT_CALLBACK                ADCSnapshotCallback;
    COH_G65DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    COH_G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    COH_G65DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
} COH_G65DDC_CALLBACKS;

typedef struct
{
    UINT32              DeviceCount;
    G65DDC_DEVICE_INFO  *DeviceInfo;
} COH_G65DDC_DEVICE_SET;

typedef struct
{
    UINT32                  DeviceSetCount;
    COH_G65DDC_DEVICE_SET   *DeviceSet;    
} COH_G65DDC_DEVICE_SET_LIST;

#pragma pack(pop)

typedef COH_G65DDC_DEVICE_SET_LIST* (__stdcall *COH_G65DDC_GET_DEVICE_SET_LIST)(void);
typedef void (__stdcall *COH_G65DDC_FREE_DEVICE_SET_LIST)(COH_G65DDC_DEVICE_SET_LIST *DeviceSetList);
typedef INT32 (__stdcall *COH_G65DDC_OPEN_DEVICE_SET)(G65DDC_DEVICE_INFO *DeviceInfos,UINT32 Count);
typedef BOOL (__stdcall *COH_G65DDC_CLOSE_DEVICE_SET)(INT32 hDeviceSet);
typedef BOOL (__stdcall *COH_G65DDC_SET_CALLBACKS)(INT32 hDeviceSet,const COH_G65DDC_CALLBACKS *Callbacks,DWORD_PTR UserData);
typedef BOOL (__stdcall *COH_G65DDC_IS_CONNECTED)(INT32 hDeviceSet,BOOL *Connected);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEVICE_COUNT)(INT32 hDeviceSet,UINT32 *Count);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEVICE_INFO)(INT32 hDeviceSet,UINT32 DeviceIndex,G65DDC_DEVICE_INFO *DeviceInfo);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEVICE_INFOS)(INT32 hDeviceSet,G65DDC_DEVICE_INFO *Infos,UINT32 *InfoCount);
typedef BOOL (__stdcall *COH_G65DDC_SET_LED)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 LEDMode);
typedef BOOL (__stdcall *COH_G65DDC_GET_LED)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *LEDMode);
typedef BOOL (__stdcall *COH_G65DDC_SET_POWER)(INT32 hDeviceSet,BOOL Power);
typedef BOOL (__stdcall *COH_G65DDC_GET_POWER)(INT32 hDeviceSet,BOOL *Power);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEVICE_STATE)(INT32 hDeviceSet,UINT32 DeviceIndex,G65DDC_DEVICE_STATE *State);
typedef BOOL (__stdcall *COH_G65DDC_SET_BUILT_IN_TEST)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_BUILT_IN_TEST)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_SET_ATTENUATOR)(INT32 hDeviceSet,UINT32 Attenuator);
typedef BOOL (__stdcall *COH_G65DDC_GET_ATTENUATOR)(INT32 hDeviceSet,UINT32 *Attenuator);
typedef BOOL (__stdcall *COH_G65DDC_SET_PRESELECTORS)(INT32 hDeviceSet,UINT32 Low,UINT32 High);
typedef BOOL (__stdcall *COH_G65DDC_GET_PRESELECTORS)(INT32 hDeviceSet,UINT32 *Low,UINT32 *High);
typedef BOOL (__stdcall *COH_G65DDC_SET_PREAMPLIFIER)(INT32 hDeviceSet,BOOL Preamp);
typedef BOOL (__stdcall *COH_G65DDC_GET_PREAMPLIFIER)(INT32 hDeviceSet,BOOL *Preamp);
typedef BOOL (__stdcall *COH_G65DDC_SET_INVERTED)(INT32 hDeviceSet,BOOL Inverted);
typedef BOOL (__stdcall *COH_G65DDC_GET_INVERTED)(INT32 hDeviceSet,BOOL *Inverted);
typedef BOOL (__stdcall *COH_G65DDC_SET_RANGE)(INT32 hDeviceSet,UINT32 Range);
typedef BOOL (__stdcall *COH_G65DDC_GET_RANGE)(INT32 hDeviceSet,UINT32 *Range);
typedef BOOL (__stdcall *COH_G65DDC_SET_ADC_NOISE_BLANKER)(INT32 hDeviceSet,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_ADC_NOISE_BLANKER)(INT32 hDeviceSet,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(INT32 hDeviceSet,WORD Threshold);
typedef BOOL (__stdcall *COH_G65DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(INT32 hDeviceSet,WORD *Threshold);
typedef BOOL (__stdcall *COH_G65DDC_START_ADC_SNAPSHOTS)(INT32 hDeviceSet,UINT32 DeviceIndex,WORD Interval,UINT32 SamplesPerSnapshot);
typedef BOOL (__stdcall *COH_G65DDC_STOP_ADC_SNAPSHOTS)(INT32 hDeviceSet,UINT32 DeviceIndex);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC_INFO)(INT32 hDeviceSet,UINT32 DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo); 
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC1_COUNT)(INT32 hDeviceSet,UINT32 *Count);
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC1)(INT32 hDeviceSet,UINT32 DDCTypeIndex);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC1)(INT32 hDeviceSet,UINT32 *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef BOOL (__stdcall *COH_G65DDC_START_DDC1)(INT32 hDeviceSet,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *COH_G65DDC_STOP_DDC1)(INT32 hDeviceSet);
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC1_FREQUENCY)(INT32 hDeviceSet,INT32 Frequency);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC1_FREQUENCY)(INT32 hDeviceSet,INT32 *Frequency);
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC1_PHASE_SHIFT)(INT32 hDeviceSet,UINT32 DeviceIndex,double PhaseShift);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC1_PHASE_SHIFT)(INT32 hDeviceSet,UINT32 DeviceIndex,double *PhaseShift);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC2_COUNT)(INT32 hDeviceSet,UINT32 *Count);
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC2)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 DDCTypeIndex);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC2)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef BOOL (__stdcall *COH_G65DDC_START_DDC2)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *COH_G65DDC_STOP_DDC2)(INT32 hDeviceSet,UINT32 DeviceIndex); 
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC2_FREQUENCY)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 Frequency);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC2_FREQUENCY)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 *Frequency);
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC2_NOISE_BLANKER)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC2_NOISE_BLANKER)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_SET_DDC2_NOISE_BLANKER_THRESHOLD)(INT32 hDeviceSet,UINT32 DeviceIndex,double Threshold);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC2_NOISE_BLANKER_THRESHOLD)(INT32 hDeviceSet,UINT32 DeviceIndex,double *Threshold);
typedef BOOL (__stdcall *COH_G65DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(INT32 hDeviceSet,UINT32 DeviceIndex,double *Value);
typedef BOOL (__stdcall *COH_G65DDC_SET_NOTCH_FILTER)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_NOTCH_FILTER)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_SET_NOTCH_FILTER_FREQUENCY)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,INT32 Frequency);
typedef BOOL (__stdcall *COH_G65DDC_GET_NOTCH_FILTER_FREQUENCY)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,INT32 *Frequency);
typedef BOOL (__stdcall *COH_G65DDC_SET_NOTCH_FILTER_BANDWIDTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 Bandwidth);
typedef BOOL (__stdcall *COH_G65DDC_GET_NOTCH_FILTER_BANDWIDTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 *Bandwidth);
typedef BOOL (__stdcall *COH_G65DDC_SET_NOTCH_FILTER_LENGTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 Length);
typedef BOOL (__stdcall *COH_G65DDC_GET_NOTCH_FILTER_LENGTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 NotchFilterIndex,UINT32 *Length);
typedef BOOL (__stdcall *COH_G65DDC_SET_AGC)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_AGC)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_SET_AGC_PARAMS)(INT32 hDeviceSet,UINT32 DeviceIndex,double AttackTime,double DecayTime,double ReferenceLevel);
typedef BOOL (__stdcall *COH_G65DDC_GET_AGC_PARAMS)(INT32 hDeviceSet,UINT32 DeviceIndex,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef BOOL (__stdcall *COH_G65DDC_SET_MAX_AGC_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double MaxGain);
typedef BOOL (__stdcall *COH_G65DDC_GET_MAX_AGC_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double *MaxGain);
typedef BOOL (__stdcall *COH_G65DDC_SET_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double Gain);
typedef BOOL (__stdcall *COH_G65DDC_GET_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double *Gain);
typedef BOOL (__stdcall *COH_G65DDC_GET_CURRENT_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double *Gain);
typedef BOOL (__stdcall *COH_G65DDC_GET_SIGNAL_LEVEL)(INT32 hDeviceSet,UINT32 DeviceIndex,float *Peak,float *RMS);
typedef BOOL (__stdcall *COH_G65DDC_SET_DEMODULATOR_MODE)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Mode);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMODULATOR_MODE)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *Mode);
typedef BOOL (__stdcall *COH_G65DDC_SET_DEMODULATOR_FREQUENCY)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 Frequency);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMODULATOR_FREQUENCY)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 *Frequency);
typedef BOOL (__stdcall *COH_G65DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Bandwidth);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *Bandwidth);
typedef BOOL (__stdcall *COH_G65DDC_SET_DEMOUDLATOR_FILTER_SHIFT)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 Shift);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMOUDLATOR_FILTER_SHIFT)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 *Shift);
typedef BOOL (__stdcall *COH_G65DDC_SET_DEMOUDLATOR_FILTER_LENGTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Length);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMOUDLATOR_FILTER_LENGTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *Length);
typedef BOOL (__stdcall *COH_G65DDC_SET_DEMODULATOR_PARAM)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Code,const void *Buffer,UINT32 BufferSize);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMODULATOR_PARAM)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Code,void *Buffer,UINT32 BufferSize);
typedef BOOL (__stdcall *COH_G65DDC_GET_DEMODULATOR_STATE)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Code,void *Buffer,UINT32 BufferSize);
typedef BOOL (__stdcall *COH_G65DDC_START_AUDIO)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *COH_G65DDC_STOP_AUDIO)(INT32 hDeviceSet,UINT32 DeviceIndex); 
typedef BOOL (__stdcall *COH_G65DDC_SET_AUDIO_FILTER)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_AUDIO_FILTER)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_SET_AUDIO_FILTER_PARAMS)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 CutOffLow,UINT32 CutOffHigh,double Deemphasis);
typedef BOOL (__stdcall *COH_G65DDC_GET_AUDIO_FILTER_PARAMS)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *CutOffLow,UINT32 *CutOffHigh,double *Deemphasis);
typedef BOOL (__stdcall *COH_G65DDC_SET_AUDIO_FILTER_LENGTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 Length);
typedef BOOL (__stdcall *COH_G65DDC_GET_AUDIO_FILTER_LENGTH)(INT32 hDeviceSet,UINT32 DeviceIndex,UINT32 *Length);
typedef BOOL (__stdcall *COH_G65DDC_SET_AUDIO_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double Gain);
typedef BOOL (__stdcall *COH_G65DDC_GET_AUDIO_GAIN)(INT32 hDeviceSet,UINT32 DeviceIndex,double *Gain);
typedef BOOL (__stdcall *COH_G65DDC_SET_VOLUME)(INT32 hDeviceSet,UINT32 DeviceIndex,BYTE Volume);
typedef BOOL (__stdcall *COH_G65DDC_GET_VOLUME)(INT32 hDeviceSet,UINT32 DeviceIndex,BYTE *Volume);
typedef BOOL (__stdcall *COH_G65DDC_SET_MUTE)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL Mute);
typedef BOOL (__stdcall *COH_G65DDC_GET_MUTE)(INT32 hDeviceSet,UINT32 DeviceIndex,BOOL *Mute);
typedef BOOL (__stdcall *COH_G65DDC_SET_DRM_KEY)(INT32 hDeviceSet,const char *DRMKeyFileDirectory);
typedef BOOL (__stdcall *COH_G65DDC_IS_DRM_UNLOCKED)(INT32 hDeviceSet);
typedef BOOL (__stdcall *COH_G65DDC_GET_SPECTRUM_COMPENSATION)(INT32 hDeviceSet,UINT32 DeviceIndex,INT32 Frequency,UINT32 Bandwidth,float *Buffer,UINT32 Count);
typedef BOOL (__stdcall *COH_G65DDC_SET_1PPS_ENABLED)(INT32 hDeviceSet,BOOL Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_1PPS_ENABLED)(INT32 hDeviceSet,BOOL *Enabled);
typedef BOOL (__stdcall *COH_G65DDC_GET_1PPS_COUNTERS)(INT32 hDeviceSet,UINT32 *EventCounter,UINT32 *ADCPeriodCounter);
typedef BOOL (__stdcall *COH_G65DDC_GET_1PPS_DDC1_COUNTERS)(INT32 hDeviceSet,double *SampleCounter,UINT64 *ADCPeriodCounter);
typedef BOOL (__stdcall *COH_G65DDC_SET_TRIGGER_DELAY)(INT32 hDeviceSet,UINT32 Delay);
typedef BOOL (__stdcall *COH_G65DDC_GET_TRIGGER_DELAY)(INT32 hDeviceSet,UINT32 *Delay);
typedef BOOL (__stdcall *COH_G65DDC_SET_CONFIGURATION)(INT32 hDeviceSet,UINT32 Configuration);
typedef BOOL (__stdcall *COH_G65DDC_GET_CONFIGURATION)(INT32 hDeviceSet,UINT32 *Configuration);
typedef BOOL (__stdcall *COH_G65DDC_SET_EXTENDED_CONFIGURATION_PARAMS)(INT32 hDeviceSet,UINT32 Index,UINT32 DeviceCount);
typedef BOOL (__stdcall *COH_G65DDC_GET_EXTENDED_CONFIGURATION_PARAMS)(INT32 hDeviceSet,UINT32 *Index,UINT32 *DeviceCount);
typedef BOOL (__stdcall *COH_G65DDC_TRIGGER)(INT32 hDeviceSet);
typedef BOOL (__stdcall *COH_G65DDC_TRIGGER_TEST)(INT32 hDeviceSet);

#endif //__G65DDCAPI_H__
